@@ -17,7 +17,7 @@
 #include "fio.h"
 #include "parse.h"
 
-static char fio_version_string[] = "fio 1.16.9";
+static char fio_version_string[] = "fio 1.17";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -222,9 +222,6 @@ static int fixup_options(struct thread_data *td)
 	if (!o->file_size_high)
 		o->file_size_high = o->file_size_low;
 
-	if (td_read(td) && !td_rw(td))
-		o->verify = 0;
-
 	if (o->norandommap && o->verify != VERIFY_NONE) {
 		log_err("fio: norandommap given, verify disabled\n");
 		o->verify = VERIFY_NONE;
