@@ -8,6 +8,7 @@
 
 #include "fio.h"
 #include "parse.h"
+#include "fls.h"
 
 #define td_var_offset(var)	((size_t) &((struct thread_options *)0)->var)
 
@@ -203,6 +204,52 @@ static int str_opendir_cb(void *data, const char fio_unused *str)
 	return add_dir_files(td, td->o.opendir);
 }
 
+static int str_verify_offset_cb(void *data, unsigned int *off)
+{
+	struct thread_data *td = data;
+
+	if (*off && *off < sizeof(struct verify_header)) {
+		log_err("fio: verify_offset too small\n");
+		return 1;
+	}
+
+	td->o.verify_offset = *off;
+	return 0;
+}
+
+static int str_verify_cb(void *data, const char *mem)
+{
+	struct thread_data *td = data;
+	unsigned int nr, msb;
+	char *pat;
+
+	if (td->o.verify != VERIFY_PATTERN)
+		return 0;
+
+	pat = get_opt_postfix(mem);
+	if (!pat) {
+		log_err("fio: missing pattern\n");
+		return 1;
+	}
+
+	if (strstr(pat, "0x") || strstr(pat, "0X"))
+		nr = strtol(pat, NULL, 16);
+	else
+		nr = strtol(pat, NULL, 16);
+
+	msb = fls(nr);
+	if (msb <= 8)
+		td->o.verify_pattern_bytes = 1;
+	else if (msb <= 16)
+		td->o.verify_pattern_bytes = 2;
+	else if (msb <= 24)
+		td->o.verify_pattern_bytes = 3;
+	else
+		td->o.verify_pattern_bytes = 4;
+
+	td->o.verify_pattern = nr;
+	return 0;
+}
 
 #define __stringify_1(x)	#x
 #define __stringify(x)		__stringify_1(x)
@@ -279,13 +326,6 @@ static struct fio_option options[] = {
 			  },
 		},
 	},
-	{
-		.name	= "fadvise_hint",
-		.type	= FIO_OPT_BOOL,
-		.off1	= td_var_offset(fadvise_hint),
-		.help	= "Use fadvise() to advise the kernel on IO pattern",
-		.def	= "1",
-	},
 	{
 		.name	= "ioengine",
 		.type	= FIO_OPT_STR_STORE,
@@ -358,12 +398,14 @@ static struct fio_option options[] = {
 		.type	= FIO_OPT_INT,
 		.off1	= td_var_offset(iodepth_batch),
 		.help	= "Number of IO to submit in one go",
+		.parent	= "iodepth",
 	},
 	{
 		.name	= "iodepth_low",
 		.type	= FIO_OPT_INT,
 		.off1	= td_var_offset(iodepth_low),
 		.help	= "Low water mark for queuing depth",
+		.parent	= "iodepth",
 	},
 	{
 		.name	= "size",
@@ -380,6 +422,14 @@ static struct fio_option options[] = {
 		.minval = 1,
 		.help	= "Size of individual files",
 	},
+	{
+		.name	= "offset",
+		.alias	= "fileoffset",
+		.type	= FIO_OPT_STR_VAL,
+		.off1	= td_var_offset(start_offset),
+		.help	= "Start IO from this offset",
+		.def	= "0",
+	},
 	{
 		.name	= "bs",
 		.alias	= "blocksize",
@@ -389,6 +439,7 @@ static struct fio_option options[] = {
 		.minval = 1,
 		.help	= "Block size unit",
 		.def	= "4k",
+		.parent = "rw",
 	},
 	{
 		.name	= "bsrange",
@@ -400,6 +451,7 @@ static struct fio_option options[] = {
 		.off4	= td_var_offset(max_bs[DDIR_WRITE]),
 		.minval = 1,
 		.help	= "Set block size range (in more detail than bs)",
+		.parent = "rw",
 	},
 	{
 		.name	= "bs_unaligned",
@@ -407,13 +459,7 @@ static struct fio_option options[] = {
 		.type	= FIO_OPT_STR_SET,
 		.off1	= td_var_offset(bs_unaligned),
 		.help	= "Don't sector align IO buffer sizes",
-	},
-	{
-		.name	= "offset",
-		.type	= FIO_OPT_STR_VAL,
-		.off1	= td_var_offset(start_offset),
-		.help	= "Start IO from this offset",
-		.def	= "0",
+		.parent = "rw",
 	},
 	{
 		.name	= "randrepeat",
@@ -421,12 +467,14 @@ static struct fio_option options[] = {
 		.off1	= td_var_offset(rand_repeatable),
 		.help	= "Use repeatable random IO pattern",
 		.def	= "1",
+		.parent = "rw",
 	},
 	{
 		.name	= "norandommap",
 		.type	= FIO_OPT_STR_SET,
 		.off1	= td_var_offset(norandommap),
 		.help	= "Accept potential duplicate random blocks",
+		.parent = "rw",
 	},
 	{
 		.name	= "nrfiles",
@@ -458,6 +506,14 @@ static struct fio_option options[] = {
 			    .help = "Round robin select files",
 			  },
 		},
+		.parent = "nrfiles",
+	},
+	{
+		.name	= "fadvise_hint",
+		.type	= FIO_OPT_BOOL,
+		.off1	= td_var_offset(fadvise_hint),
+		.help	= "Use fadvise() to advise the kernel on IO pattern",
+		.def	= "1",
 	},
 	{
 		.name	= "fsync",
@@ -562,6 +618,7 @@ static struct fio_option options[] = {
 		.name	= "verify",
 		.type	= FIO_OPT_STR,
 		.off1	= td_var_offset(verify),
+		.cb	= str_verify_cb,
 		.help	= "Verify data written",
 		.def	= "0",
 		.posval = {
@@ -569,13 +626,41 @@ static struct fio_option options[] = {
 			    .oval = VERIFY_NONE,
 			    .help = "Don't do IO verification",
 			  },
+			  { .ival = "md5",
+			    .oval = VERIFY_MD5,
+			    .help = "Use md5 checksums for verification",
+			  },
+			  { .ival = "crc64",
+			    .oval = VERIFY_CRC64,
+			    .help = "Use crc64 checksums for verification",
+			  },
 			  { .ival = "crc32",
 			    .oval = VERIFY_CRC32,
 			    .help = "Use crc32 checksums for verification",
 			  },
-			  { .ival = "md5",
-			    .oval = VERIFY_MD5,
-			    .help = "Use md5 checksums for verification",
+			  { .ival = "crc16",
+			    .oval = VERIFY_CRC16,
+			    .help = "Use crc16 checksums for verification",
+			  },
+			  { .ival = "crc7",
+			    .oval = VERIFY_CRC7,
+			    .help = "Use crc7 checksums for verification",
+			  },
+			  { .ival = "sha256",
+			    .oval = VERIFY_SHA256,
+			    .help = "Use sha256 checksums for verification",
+			  },
+			  { .ival = "sha512",
+			    .oval = VERIFY_SHA512,
+			    .help = "Use sha512 checksums for verification",
+			  },
+			  { .ival = "meta",
+			    .oval = VERIFY_META,
+			    .help = "Use io information",
+			  },
+			  { .ival = "pattern",
+			    .oval = VERIFY_PATTERN,
+			    .help = "Verify a specific buffer pattern",
 			  },
 			  {
 			    .ival = "null",
@@ -584,12 +669,45 @@ static struct fio_option options[] = {
 			  },
 		},
 	},
+	{
+		.name	= "do_verify",
+		.type	= FIO_OPT_BOOL,
+		.off1	= td_var_offset(do_verify),
+		.help	= "Run verification stage after write",
+		.def	= "1",
+		.parent = "verify",
+	},
 	{
 		.name	= "verifysort",
 		.type	= FIO_OPT_BOOL,
 		.off1	= td_var_offset(verifysort),
 		.help	= "Sort written verify blocks for read back",
 		.def	= "1",
+		.parent = "verify",
+	},
+	{
+		.name   = "verify_interval",
+		.type   = FIO_OPT_STR_VAL_INT,
+		.off1   = td_var_offset(verify_interval),
+		.minval	= 2 * sizeof(struct verify_header),
+		.help   = "Store verify buffer header every N bytes",
+		.parent	= "verify",
+	},
+	{
+		.name	= "verify_offset",
+		.type	= FIO_OPT_STR_VAL_INT,
+		.help	= "Offset verify header location by N bytes",
+		.def	= "0",
+		.cb	= str_verify_offset_cb,	
+		.parent	= "verify",
+	},
+	{
+		.name	= "verify_fatal",
+		.type	= FIO_OPT_BOOL,
+		.off1	= td_var_offset(verify_fatal),
+		.def	= "0",
+		.help	= "Exit on a single verify failure, don't continue",
+		.parent = "verify",
 	},
 	{
 		.name	= "write_iolog",
@@ -644,13 +762,6 @@ static struct fio_option options[] = {
 		.help	= "Lock down this amount of memory",
 		.def	= "0",
 	},
-	{
-		.name	= "rwmixcycle",
-		.type	= FIO_OPT_INT,
-		.off1	= td_var_offset(rwmixcycle),
-		.help	= "Cycle period for mixed read/write workloads (msec)",
-		.def	= "500",
-	},
 	{
 		.name	= "rwmixread",
 		.type	= FIO_OPT_INT,
@@ -667,6 +778,14 @@ static struct fio_option options[] = {
 		.help	= "Percentage of mixed workload that is writes",
 		.def	= "50",
 	},
+	{
+		.name	= "rwmixcycle",
+		.type	= FIO_OPT_INT,
+		.off1	= td_var_offset(rwmixcycle),
+		.help	= "Cycle period for mixed read/write workloads (msec)",
+		.def	= "500",
+		.parent	= "rwmixread",
+	},
 	{
 		.name	= "nice",
 		.type	= FIO_OPT_INT,
@@ -707,6 +826,7 @@ static struct fio_option options[] = {
 		.off1	= td_var_offset(thinktime_spin),
 		.help	= "Start think time by spinning this amount (usec)",
 		.def	= "0",
+		.parent	= "thinktime",
 	},
 	{
 		.name	= "thinktime_blocks",
@@ -714,6 +834,7 @@ static struct fio_option options[] = {
 		.off1	= td_var_offset(thinktime_blocks),
 		.help	= "IO buffer period between 'thinktime'",
 		.def	= "1",
+		.parent	= "thinktime",
 	},
 	{
 		.name	= "rate",
@@ -726,6 +847,7 @@ static struct fio_option options[] = {
 		.type	= FIO_OPT_INT,
 		.off1	= td_var_offset(ratemin),
 		.help	= "Job must meet this rate or it will be shutdown",
+		.parent	= "rate",
 	},
 	{
 		.name	= "rate_iops",
@@ -738,6 +860,7 @@ static struct fio_option options[] = {
 		.type	= FIO_OPT_INT,
 		.off1	= td_var_offset(rate_iops_min),
 		.help	= "Job must meet this rate or it will be shutdown",
+		.parent	= "rate_iops",
 	},
 	{
 		.name	= "ratecycle",
@@ -745,6 +868,7 @@ static struct fio_option options[] = {
 		.off1	= td_var_offset(ratecycle),
 		.help	= "Window average for rate limits (msec)",
 		.def	= "1000",
+		.parent = "rate",
 	},
 	{
 		.name	= "invalidate",
@@ -759,6 +883,7 @@ static struct fio_option options[] = {
 		.off1	= td_var_offset(sync_io),
 		.help	= "Use O_SYNC for buffered writes",
 		.def	= "0",
+		.parent = "buffered",
 	},
 	{
 		.name	= "bwavgtime",
@@ -793,6 +918,7 @@ static struct fio_option options[] = {
 		.off1	= td_var_offset(cpucycle),
 		.help	= "Length of the CPU burn cycles (usecs)",
 		.def	= "50000",
+		.parent = "cpuload",
 	},
 #ifdef FIO_HAVE_CPU_AFFINITY
 	{
