@@ -26,19 +26,30 @@
 
 #define OS_MAP_ANON		(MAP_ANONYMOUS)
 
+#ifndef CLOCK_MONOTONIC
+#define CLOCK_MONOTONIC 1
+#endif
+
+#ifdef FIO_HAVE_CPU_AFFINITY
 typedef cpu_set_t os_cpu_mask_t;
+#else
+typedef int os_cpu_mask_t;
+#endif
 typedef struct drand48_data os_random_state_t;
 
 /*
  * we want fadvise64 really, but it's so tangled... later
  */
+#ifdef FIO_HAVE_FADVISE
 #define fadvise(fd, off, len, advice)	\
 	posix_fadvise((fd), (off_t)(off), (len), (advice))
+#endif
 
 /*
  * If you are on an ancient glibc (2.3.2), then define GLIBC_2_3_2 if you want
  * the affinity helpers to work.
  */
+#ifdef FIO_HAVE_CPU_AFFINITY
 #ifndef GLIBC_2_3_2
 #define fio_setaffinity(td)		\
 	sched_setaffinity((td)->pid, sizeof((td)->o.cpumask), &(td)->o.cpumask)
@@ -50,6 +61,7 @@ typedef struct drand48_data os_random_state_t;
 #define fio_getaffinity(pid, ptr)	\
 	sched_getaffinity((pid), (ptr))
 #endif
+#endif
 
 static inline int ioprio_set(int which, int who, int ioprio)
 {
@@ -69,7 +81,7 @@ static inline int ioprio_set(int which, int who, int ioprio)
 #define SPLICE_F_GIFT   (0x08)  /* pages passed in are a gift */
 
 static inline int splice(int fdin, loff_t *off_in, int fdout, loff_t *off_out,
-			 size_t len, unsigned long flags)
+			 size_t len, unsigned int flags)
 {
 	return syscall(__NR_sys_splice, fdin, off_in, fdout, off_out, len, flags);
 }
@@ -86,6 +98,10 @@ static inline int vmsplice(int fd, const struct iovec *iov,
 }
 #endif
 
+#ifdef SPLICE_F_UNMAP
+#define SPLICE_F_UNMAP	(0x10)
+#endif
+
 #define SPLICE_DEF_SIZE	(64*1024)
 
 #ifdef FIO_HAVE_SYSLET
@@ -183,14 +199,6 @@ static inline long os_random_long(os_random_state_t *rs)
 	return val;
 }
 
-static inline double os_random_double(os_random_state_t *rs)
-{
-	double val;
-
-	drand48_r(rs, &val);
-	return val;
-}
-
 static inline int fio_lookup_raw(dev_t dev, int *majdev, int *mindev)
 {
 	struct raw_config_request rq;
